#!/usr/bin/env python
# conversion_test.py

import string
import unittest

import sapdb

class ConversionTest (unittest.TestCase):
    def createTable (self, tableName, definition):
        try:
            self.session.sql ('drop table ' + tableName)
        except sapdb.SQLError:
            pass
        self.session.sql ('create table %s (%s)' % (tableName, definition))

    def insertSequential (self, cmd, values, isPrepared = None):
        if isPrepared:
            insert = cmd
        else:
            insert = self.session.prepare (cmd)
        for tuple in values:
            insert.execute (tuple)
        self.session.commit ()

    def setUp (self):
        self.session = sapdb.connect ('SUT',  'SUT',  'SUT73', 'p48047')
        self.createTable ('currency', "keycol serial key, dm fixed (38, 4)")

    def tearDown (self):
        del self.session

    def __del__ (self):
        if hasattr (self, 'session'):
            del self.session

    def normalizeNumberstring (self, value, frac):
        value = str (value)
        parts = string.split (value,  '.')
        if len (parts [1]) < frac:
            missing = frac - len (parts [1])
            parts [1] = parts [1] + ('0' * missing)
            return string.join (parts,  '.')
        else:
            return value


class FetchConversion (ConversionTest):

    moneyValues = [(37.4,)]

    def setUp (self):
        ConversionTest.setUp (self)

    def getCursor (self, columns = ['dm']):
        colstr = string.join (columns,  ',  ')
        return self.session.sql ('select ' + colstr + ' from currency order by keycol')

    def testBare (self):
        cursor = self.getCursor ()
        for expected, actual in ZipIterator (self.moneyValues, cursor):
            self.assertEqual (expected, actual)

    def testByCol (self):
        self.insertSequential ('insert into currency (dm) values (?)', self.moneyValues)
        #
        # check error handling
        #
        cursor = self.getCursor ()
        self.assertRaises (sapdb.SQLError, cursor.setFetchConversionByCol,
            1, "s", dollar)
        self.assertRaises (sapdb.SQLError, cursor.setFetchConversionByCol,
            0, "x", dollar)
        self.assertRaises (sapdb.SQLError, cursor.setFetchConversionByCol,
            0, "s", "some string")
        #
        # check conversion without callback
        #
        cursor = self.getCursor ()
        cursor.setFetchConversionByCol (0, "s")
        for (expected,), (actual,) in ZipIterator (self.moneyValues, cursor):
            expected = self.normalizeNumberstring (expected, 4)
            self.assertEqual (expected, actual)
        #
        # check conversion with callback
        #
        cursor = self.getCursor ()
        cursor.setFetchConversionByCol (0, "s", dollar)
        for (expected,), (actual,) in ZipIterator (self.moneyValues, cursor):
            expected = '$ ' + self.normalizeNumberstring (expected, 4)
            self.assertEqual (expected, actual)

    def testByType (self):
        self.insertSequential ('insert into currency (dm) values (?)', self.moneyValues)
        #
        # check error handling
        #
        cursor = self.getCursor ()
        self.assertRaises (sapdb.SQLError, cursor.setFetchConversionByType,
            "garbage", "s", dollar)
        self.assertRaises (sapdb.SQLError, cursor.setFetchConversionByType,
            "Fixed", "x", dollar)
        self.assertRaises (sapdb.SQLError, cursor.setFetchConversionByType,
            "Fixed", "s", "some string")
        #
        # check conversion without callback
        #
        cursor = self.getCursor (['dm', "'hello'"])
        cursor.setFetchConversionByType ('Fixed', "s")
        for (expecteddm,), (actualdm, actualhello) in ZipIterator (self.moneyValues, cursor):
            expecteddm = str (expecteddm)
            self.assertEqual (self.normalizeNumberstring (expecteddm, 4), actualdm)
            self.assertEqual ('hello', actualhello)
        #
        # check conversion with callback
        #
        cursor = self.getCursor (['dm', "'hello'"])
        cursor.setFetchConversionByType ('Fixed', "s")
        for (expecteddm,), (actualdm, actualhello) in ZipIterator (self.moneyValues, cursor):
            expecteddm = self.normalizeNumberstring (expecteddm, 4)
            self.assertEqual (expecteddm, actualdm)
            self.assertEqual ('hello', actualhello)


    def testMantisseExponent (self):
        moneyValues = [(37.4,), (0,), (0.01,), (-2,), (-0.01,)]
        self.insertSequential ('insert into currency (dm) values (?)', moneyValues)
        #
        # check conversion without callback
        #
        expectedValues = [((374, 2),), ((0, 0),), ((1, -1),), ((-2, 1),), ((-1, -1),), ]
        cursor = self.getCursor ()
        cursor.setFetchConversionByCol (0, "me")
        for (expected,), (actual,) in ZipIterator (expectedValues, cursor):
            self.assertEqual (expected, actual)
        #
        # check conversion with callback
        #
        expectedValues = [("0.374E2",), ("0.0E0",), ("0.1E-1",), ("-0.2E1",), ("-0.1E-1",), ]
        cursor = self.getCursor ()
        def format (mantisse, exponent):
            if mantisse < 0:
                result = "-0.%dE%d" % (-mantisse, exponent)
            else:
                result = "0.%dE%d" % (mantisse, exponent)
            return result

        cursor.setFetchConversionByCol (0, "me", format)
        for (expected,), (actual,) in ZipIterator (expectedValues, cursor):
            self.assertEqual (expected, actual)

def dollar (str):
    return '$ ' + str

class InsertConversion (ConversionTest):


    def testBase (self):
        moneyValues = [(37.4,),]
        self.insertSequential ('insert into currency (dm) values (?)', moneyValues)
        cursor = self.session.sql ('select chr (dm) from currency order by keycol')
        for (expected,), (actual,) in ZipIterator (moneyValues, cursor):
            expected = self.normalizeNumberstring (expected, 4)
            self.assertEqual (expected, actual)

    def testByCol (self):
        moneyValues = [('37.4',),]
        insert = self.session.prepare ('insert into currency (dm) values (?)')
        #
        # test conversion from string only
        #
        insert.setParameterConversionByCol (0, "s")
        self.insertSequential (insert, moneyValues, 1)
        cursor = self.session.sql ('select chr (dm) from currency order by keycol')
        for (expected,), (actual,) in ZipIterator (moneyValues, cursor):
            expected = self.normalizeNumberstring (expected, 4)
            self.assertEqual (expected, actual)

        #
        # test conversion + callback
        #
        self.session.sql ('delete from currency')
        moneyValues = [('$ 37.4',),]
        def stripDollar (str):
            return str [2:]
        insert.setParameterConversionByCol (0, "s", stripDollar)
        self.insertSequential (insert, moneyValues, 1)
        cursor = self.session.sql ('select chr (dm) from currency order by keycol')
        for (expected,), (actual,) in ZipIterator (moneyValues, cursor):
            expected = self.normalizeNumberstring (stripDollar (expected), 4)
            self.assertEqual (expected, actual)

    def testByType (self):
        moneyValues = [('37.4',),]
        insert = self.session.prepare ('insert into currency (dm) values (?)')
        insert.setParameterConversionByType ("Fixed", "s")
        self.insertSequential (insert, moneyValues, 1)
        cursor = self.session.sql ('select chr (dm) from currency order by keycol')
        for (expected,), (actual,) in ZipIterator (moneyValues, cursor):
            expected = self.normalizeNumberstring (expected, 4)
            self.assertEqual (expected, actual)

    def testMantisseExponent (self):
        moneyValues =    [((374, 2),),  ((0, 0),),  ((1, -1),), ((-2, 1),),  ((-1, -1),), ]
        expectedValues = [("37.4000",), ("0.0000",), ("0.0100",),("-2.0000",), ("-0.0100",), ]
        #
        # conversion without callback
        #
        insert = self.session.prepare ('insert into currency (dm) values (?)')
        insert.setParameterConversionByType ("Fixed", "me")
        self.insertSequential (insert, moneyValues, 1)
        cursor = self.session.sql ('select chr (dm) from currency order by keycol')
        for (expected,), (actual,) in ZipIterator (expectedValues, cursor):
            self.assertEqual (expected, actual)
        #
        # conversion with callback
        #
        insert = self.session.prepare ('insert into currency (dm) values (?)')
        def identity (x):
            return x
        insert.setParameterConversionByType ("Fixed", "me", identity)
        self.insertSequential (insert, moneyValues, 1)
        cursor = self.session.sql ('select chr (dm) from currency order by keycol')
        for (expected,), (actual,) in ZipIterator (expectedValues, cursor):
            self.assertEqual (expected, actual)

class IndexedIterator:
    def __init__ (self, list):
        self.list = list

    def __getitem__ (self, index):
        return index, self.list [index]

class ZipIterator:
    def __init__ (self, list0, list1):
        self.lists = [list0, list1]

    def __getitem__ (self, index):
        return (self.lists [0] [index], self.lists [1] [index])

def main (options, args):
    unittest.main ()

def _options ():
    return [
        # (optstring, varname, typechar, default, help)
        ]

if __name__ == '__main__':
    unittest.main ()
    #import optlib
    #optlib.optMain2 (main, _options ())
